package com.sun.media.sound;

import java.util.Arrays;

public final class SoftChorus implements SoftAudioProcessor
{
    private boolean mix;
    private SoftAudioBuffer inputA;
    private SoftAudioBuffer left;
    private SoftAudioBuffer right;
    private SoftAudioBuffer reverb;
    private SoftChorus.LFODelay vdelay1L;
    private SoftChorus.LFODelay vdelay1R;
    private float rgain;
    private boolean dirty;
    private double dirty_vdelay1L_rate;
    private double dirty_vdelay1R_rate;
    private double dirty_vdelay1L_depth;
    private double dirty_vdelay1R_depth;
    private float dirty_vdelay1L_feedback;
    private float dirty_vdelay1R_feedback;
    private float dirty_vdelay1L_reverbsendgain;
    private float dirty_vdelay1R_reverbsendgain;
    private float controlrate;
    double silentcounter;
    
    public SoftChorus() {
        this.mix = true;
        this.rgain = 0.0f;
        this.dirty = true;
        this.silentcounter = 1000.0;
    }
    
    @Override
    public void init(float n, float controlrate) {
        this.controlrate = controlrate;
        this.vdelay1L = new SoftChorus.LFODelay((double)n, (double)controlrate);
        this.vdelay1R = new SoftChorus.LFODelay((double)n, (double)controlrate);
        this.vdelay1L.setGain(1.0f);
        this.vdelay1R.setGain(1.0f);
        this.vdelay1L.setPhase(1.5707963267948966);
        this.vdelay1R.setPhase(0.0);
        this.globalParameterControlChange(new int[] { 130 }, 0L, 2L);
    }
    
    @Override
    public void globalParameterControlChange(int[] array, long n, long n2) {
        if (array.length == 1 && array[0] == 130) {
            if (n == 0L) {
                switch ((int)n2) {
                    case 0: {
                        this.globalParameterControlChange(array, 3L, 0L);
                        this.globalParameterControlChange(array, 1L, 3L);
                        this.globalParameterControlChange(array, 2L, 5L);
                        this.globalParameterControlChange(array, 4L, 0L);
                        break;
                    }
                    case 1: {
                        this.globalParameterControlChange(array, 3L, 5L);
                        this.globalParameterControlChange(array, 1L, 9L);
                        this.globalParameterControlChange(array, 2L, 19L);
                        this.globalParameterControlChange(array, 4L, 0L);
                        break;
                    }
                    case 2: {
                        this.globalParameterControlChange(array, 3L, 8L);
                        this.globalParameterControlChange(array, 1L, 3L);
                        this.globalParameterControlChange(array, 2L, 19L);
                        this.globalParameterControlChange(array, 4L, 0L);
                        break;
                    }
                    case 3: {
                        this.globalParameterControlChange(array, 3L, 16L);
                        this.globalParameterControlChange(array, 1L, 9L);
                        this.globalParameterControlChange(array, 2L, 16L);
                        this.globalParameterControlChange(array, 4L, 0L);
                        break;
                    }
                    case 4: {
                        this.globalParameterControlChange(array, 3L, 64L);
                        this.globalParameterControlChange(array, 1L, 2L);
                        this.globalParameterControlChange(array, 2L, 24L);
                        this.globalParameterControlChange(array, 4L, 0L);
                        break;
                    }
                    case 5: {
                        this.globalParameterControlChange(array, 3L, 112L);
                        this.globalParameterControlChange(array, 1L, 1L);
                        this.globalParameterControlChange(array, 2L, 5L);
                        this.globalParameterControlChange(array, 4L, 0L);
                        break;
                    }
                    default: {
                        break;
                    }
                }
            }
            else if (n == 1L) {
                this.dirty_vdelay1L_rate = (double)n2 * 0.122;
                this.dirty_vdelay1R_rate = (double)n2 * 0.122;
                this.dirty = true;
            }
            else if (n == 2L) {
                this.dirty_vdelay1L_depth = (double)(n2 + 1L) / 3200.0;
                this.dirty_vdelay1R_depth = (double)(n2 + 1L) / 3200.0;
                this.dirty = true;
            }
            else if (n == 3L) {
                this.dirty_vdelay1L_feedback = (float)n2 * 0.00763f;
                this.dirty_vdelay1R_feedback = (float)n2 * 0.00763f;
                this.dirty = true;
            }
            if (n == 4L) {
                this.rgain = (float)n2 * 0.00787f;
                this.dirty_vdelay1L_reverbsendgain = (float)n2 * 0.00787f;
                this.dirty_vdelay1R_reverbsendgain = (float)n2 * 0.00787f;
                this.dirty = true;
            }
        }
    }
    
    @Override
    public void processControlLogic() {
        if (this.dirty) {
            this.dirty = false;
            this.vdelay1L.setRate(this.dirty_vdelay1L_rate);
            this.vdelay1R.setRate(this.dirty_vdelay1R_rate);
            this.vdelay1L.setDepth(this.dirty_vdelay1L_depth);
            this.vdelay1R.setDepth(this.dirty_vdelay1R_depth);
            this.vdelay1L.setFeedBack(this.dirty_vdelay1L_feedback);
            this.vdelay1R.setFeedBack(this.dirty_vdelay1R_feedback);
            this.vdelay1L.setReverbSendGain(this.dirty_vdelay1L_reverbsendgain);
            this.vdelay1R.setReverbSendGain(this.dirty_vdelay1R_reverbsendgain);
        }
    }
    
    @Override
    public void processAudio() {
        if (this.inputA.isSilent()) {
            this.silentcounter += (double)(1.0f / this.controlrate);
            if (this.silentcounter > 1.0) {
                if (!this.mix) {
                    this.left.clear();
                    this.right.clear();
                }
                return;
            }
        }
        else {
            this.silentcounter = 0.0;
        }
        float[] array = this.inputA.array();
        float[] array2 = this.left.array();
        float[] array3 = (this.right == null) ? null : this.right.array();
        float[] array4 = (this.rgain != 0.0f) ? this.reverb.array() : null;
        if (this.mix) {
            this.vdelay1L.processMix(array, array2, array4);
            if (array3 != null) {
                this.vdelay1R.processMix(array, array3, array4);
            }
        }
        else {
            this.vdelay1L.processReplace(array, array2, array4);
            if (array3 != null) {
                this.vdelay1R.processReplace(array, array3, array4);
            }
        }
    }
    
    @Override
    public void setInput(int n, SoftAudioBuffer inputA) {
        if (n == 0) {
            this.inputA = inputA;
        }
    }
    
    @Override
    public void setMixMode(boolean mix) {
        this.mix = mix;
    }
    
    @Override
    public void setOutput(int n, SoftAudioBuffer reverb) {
        if (n == 0) {
            this.left = reverb;
        }
        if (n == 1) {
            this.right = reverb;
        }
        if (n == 2) {
            this.reverb = reverb;
        }
    }
    
    private static class VariableDelay
    {
        private final float[] delaybuffer;
        private int rovepos;
        private float gain;
        private float rgain;
        private float delay;
        private float lastdelay;
        private float feedback;
        
        VariableDelay(int n) {
            this.rovepos = 0;
            this.gain = 1.0f;
            this.rgain = 0.0f;
            this.delay = 0.0f;
            this.lastdelay = 0.0f;
            this.feedback = 0.0f;
            this.delaybuffer = new float[n];
        }
        
        public void setDelay(float delay) {
            this.delay = delay;
        }
        
        public void setFeedBack(float feedback) {
            this.feedback = feedback;
        }
        
        public void setGain(float gain) {
            this.gain = gain;
        }
        
        public void setReverbSendGain(float rgain) {
            this.rgain = rgain;
        }
        
        public void processMix(float[] array, float[] array2, float[] array3) {
            float gain = this.gain;
            float delay = this.delay;
            float feedback = this.feedback;
            float[] delaybuffer = this.delaybuffer;
            int length = array.length;
            float n = (delay - this.lastdelay) / (float)length;
            int length2 = delaybuffer.length;
            int rovepos = this.rovepos;
            if (array3 == null) {
                for (int j = 0; j < length; ++j) {
                    float n8 = (float)rovepos - (this.lastdelay + 2.0f) + (float)length2;
                    int n9 = (int)n8;
                    float n10 = n8 - (float)n9;
                    float n11 = delaybuffer[n9 % length2] * (1.0f - n10) + delaybuffer[(n9 + 1) % length2] * n10;
                    int n12 = j;
                    array2[n12] += n11 * gain;
                    delaybuffer[rovepos] = array[j] + n11 * feedback;
                    rovepos = (rovepos + 1) % length2;
                    this.lastdelay += n;
                }
            }
            else {
                for (int i = 0; i < length; ++i) {
                    float n2 = (float)rovepos - (this.lastdelay + 2.0f) + (float)length2;
                    int n3 = (int)n2;
                    float n4 = n2 - (float)n3;
                    float n5 = delaybuffer[n3 % length2] * (1.0f - n4) + delaybuffer[(n3 + 1) % length2] * n4;
                    int n6 = i;
                    array2[n6] += n5 * gain;
                    int n7 = i;
                    array3[n7] += n5 * this.rgain;
                    delaybuffer[rovepos] = array[i] + n5 * feedback;
                    rovepos = (rovepos + 1) % length2;
                    this.lastdelay += n;
                }
            }
            this.rovepos = rovepos;
            this.lastdelay = delay;
        }
        
        public void processReplace(float[] array, float[] a, float[] a2) {
            Arrays.fill(a, 0.0f);
            Arrays.fill(a2, 0.0f);
            this.processMix(array, a, a2);
        }
    }
    
    private static class LFODelay
    {
        private double phase;
        private double phase_step;
        private double depth;
        private SoftChorus.VariableDelay vdelay;
        private final double samplerate;
        private final double controlrate;
        
        LFODelay(double samplerate, double controlrate) {
            this.phase = 1.0;
            this.phase_step = 0.0;
            this.depth = 0.0;
            this.samplerate = samplerate;
            this.controlrate = controlrate;
            this.vdelay = new SoftChorus.VariableDelay((int)((this.depth + 10.0) * 2.0));
        }
        
        public void setDepth(double n) {
            this.depth = n * this.samplerate;
            this.vdelay = new SoftChorus.VariableDelay((int)((this.depth + 10.0) * 2.0));
        }
        
        public void setRate(double n) {
            this.phase_step = 6.283185307179586 * (n / this.controlrate);
        }
        
        public void setPhase(double phase) {
            this.phase = phase;
        }
        
        public void setFeedBack(float feedBack) {
            this.vdelay.setFeedBack(feedBack);
        }
        
        public void setGain(float gain) {
            this.vdelay.setGain(gain);
        }
        
        public void setReverbSendGain(float reverbSendGain) {
            this.vdelay.setReverbSendGain(reverbSendGain);
        }
        
        public void processMix(float[] array, float[] array2, float[] array3) {
            this.phase += this.phase_step;
            while (this.phase > 6.283185307179586) {
                this.phase -= 6.283185307179586;
            }
            this.vdelay.setDelay((float)(this.depth * 0.5 * (Math.cos(this.phase) + 2.0)));
            this.vdelay.processMix(array, array2, array3);
        }
        
        public void processReplace(float[] array, float[] array2, float[] array3) {
            this.phase += this.phase_step;
            while (this.phase > 6.283185307179586) {
                this.phase -= 6.283185307179586;
            }
            this.vdelay.setDelay((float)(this.depth * 0.5 * (Math.cos(this.phase) + 2.0)));
            this.vdelay.processReplace(array, array2, array3);
        }
    }
}
